@@ -93,7 +93,8 @@ AbstractClient::~AbstractClient()
 struct TELEPATHY_QT4_NO_EXPORT AbstractClientObserver::Private
 {
     Private(const ChannelClassList &channelFilter, bool shouldRecover)
-        : channelFilter(channelFilter), shouldRecover(shouldRecover)
+        : channelFilter(channelFilter),
+          shouldRecover(shouldRecover)
     {
     }
 
@@ -292,7 +293,7 @@ AbstractClientObserver::~AbstractClientObserver()
  * The same principle is applied to approvers and handlers.
  *
  * \return A specification of the channels that this channel observer is
- *         interested.
+ *         interested as a list of ChannelClassSpec objects.
  * \sa observeChannels()
  */
 ChannelClassSpecList AbstractClientObserver::observerFilter() const
@@ -368,6 +369,11 @@ bool AbstractClientObserver::shouldRecover() const
 
 struct TELEPATHY_QT4_NO_EXPORT AbstractClientApprover::Private
 {
+    Private(const ChannelClassList &channelFilter)
+        : channelFilter(channelFilter)
+    {
+    }
+
     ChannelClassList channelFilter;
 };
 
@@ -480,9 +486,8 @@ struct TELEPATHY_QT4_NO_EXPORT AbstractClientApprover::Private
  */
 AbstractClientApprover::AbstractClientApprover(
         const ChannelClassSpecList &channelFilter)
-    : mPriv(new Private)
+    : mPriv(new Private(channelFilter.bareClasses()))
 {
-    mPriv->channelFilter = channelFilter.bareClasses();
 }
 
 /**
@@ -505,11 +510,11 @@ AbstractClientApprover::~AbstractClientApprover()
  * corresponding client bus name.
  *
  * In the .client file, represented in the same way as observer channel
- * filter, the group is #TELEPATHY_INTERFACE_CLIENT_APPROVER followed by
+ * filter, the group is #TP_QT4_IFACE_CLIENT_APPROVER followed by
  * ApproverChannelFilter instead.
  *
  * \return A specification of the channels that this channel approver is
- *         interested.
+ *         interested as a list of ChannelClassSpec objects.
  * \sa addDispatchOperation()
  */
 ChannelClassSpecList AbstractClientApprover::approverFilter() const
@@ -540,9 +545,20 @@ ChannelClassSpecList AbstractClientApprover::approverFilter() const
 
 struct TELEPATHY_QT4_NO_EXPORT AbstractClientHandler::Private
 {
+    Private(const ChannelClassList &channelFilter,
+            const Capabilities &capabilities,
+            bool wantsRequestNotification)
+        : channelFilter(channelFilter),
+          capabilities(capabilities),
+          wantsRequestNotification(wantsRequestNotification),
+          registered(false)
+    {
+    }
+
     ChannelClassList channelFilter;
     Capabilities capabilities;
     bool wantsRequestNotification;
+    bool registered;
 };
 
 /**
@@ -771,11 +787,8 @@ QVariantMap AbstractClientHandler::HandlerInfo::allInfo() const
 AbstractClientHandler::AbstractClientHandler(const ChannelClassSpecList &channelFilter,
         const Capabilities &capabilities,
         bool wantsRequestNotification)
-    : mPriv(new Private)
+    : mPriv(new Private(channelFilter.bareClasses(), capabilities, wantsRequestNotification))
 {
-    mPriv->channelFilter = channelFilter.bareClasses();
-    mPriv->capabilities = capabilities;
-    mPriv->wantsRequestNotification = wantsRequestNotification;
 }
 
 /**
@@ -786,6 +799,16 @@ AbstractClientHandler::~AbstractClientHandler()
     delete mPriv;
 }
 
+/**
+ * Return whether this handler is registered.
+ *
+ * \return \c true if registered, \c false otherwise.
+ */
+bool AbstractClientHandler::isRegistered() const
+{
+    return mPriv->registered;
+}
+
 /**
  * Return the property containing a specification of the channels that this
  * channel handler can deal with. It will be offered to approvers as a potential
@@ -798,11 +821,11 @@ AbstractClientHandler::~AbstractClientHandler()
  * corresponding client bus name.
  *
  * In the .client file, represented in the same way as observer channel
- * filter, the group is #TELEPATHY_INTERFACE_CLIENT_HANDLER suffixed
+ * filter, the group is #TP_QT4_IFACE_CLIENT_HANDLER suffixed
  * by HandlerChannelFilter instead.
  *
  * \return A specification of the channels that this channel handler can deal
- *         with.
+ *         with as a list of ChannelClassSpec objects.
  */
 ChannelClassSpecList AbstractClientHandler::handlerFilter() const
 {
@@ -812,7 +835,7 @@ ChannelClassSpecList AbstractClientHandler::handlerFilter() const
 /**
  * Return the set of additional capabilities supported by this handler.
  *
- * \return The set of additional capabilities supported by this handler.
+ * \return The capabilities as an AbstractClientHandler::Capabilities object.
  */
 AbstractClientHandler::Capabilities AbstractClientHandler::handlerCapabilities() const
 {
@@ -825,7 +848,7 @@ AbstractClientHandler::Capabilities AbstractClientHandler::handlerCapabilities()
  * Return whether channels destined for this handler are automatically
  * handled, without invoking approvers.
  *
- * \return Whether channels destined for this handler are automatically handled.
+ * \return \c true if automatically handled, \c false otherwise.
  */
 
 /**
@@ -886,7 +909,8 @@ AbstractClientHandler::Capabilities AbstractClientHandler::handlerCapabilities()
  *
  * This property is set by the constructor and cannot be changed after that.
  *
- * \return Whether this handler wants to receive channel requests notification.
+ * \return \c true if receiving channel requests notification is desired,
+ *         \c false otherwise.
  */
 bool AbstractClientHandler::wantsRequestNotification() const
 {
@@ -916,7 +940,7 @@ bool AbstractClientHandler::wantsRequestNotification() const
  * The channel dispatcher will attempt to ensure that handleChannels() is called
  * on the same handler that received addRequest(). If that isn't possible,
  * removeRequest() will be called on the handler that previously received
- * addRequest(), with the special error #TELEPATHY_ERROR_NOT_YOURS, which
+ * addRequest(), with the special error #TP_QT4_ERROR_NOT_YOURS, which
  * indicates that some other handler received the channel instead.
  *
  * Expected handling is for the UI to close the window it previously opened.
@@ -943,7 +967,7 @@ void AbstractClientHandler::addRequest(
  *
  * \param channelRequest The channel request that failed.
  * \param errorName The name of the D-Bus error with which the request failed.
- *                  If this is #TELEPATHY_ERROR_NOT_YOURS, this indicates that
+ *                  If this is #TP_QT4_ERROR_NOT_YOURS, this indicates that
  *                  the request succeeded, but all the resulting channels were
  *                  given to some other handler.
  * \param errorMessage Any message supplied with the D-Bus error.
@@ -956,4 +980,9 @@ void AbstractClientHandler::removeRequest(
     // this method
 }
 
+void AbstractClientHandler::setRegistered(bool registered)
+{
+    mPriv->registered = registered;
+}
+
 } // Tp
