@@ -71,7 +71,7 @@ ChannelFactory::Private::Private()
  */
 
 /**
- * Create a new ChannelFactory object for the given \a bus.
+ * Create a new ChannelFactory object.
  *
  * The returned factory will construct channel subclasses provided by TelepathyQt4 as appropriate
  * for the channel immutable properties, but not make any features ready.
@@ -85,7 +85,7 @@ ChannelFactoryPtr ChannelFactory::create(const QDBusConnection &bus)
 }
 
 /**
- * Class constructor.
+ * Construct a new ChannelFactory object.
  *
  * The constructed factory will construct channel subclasses provided by TelepathyQt4 as appropriate
  * for the channel immutable properties, but not make any features ready.
@@ -103,6 +103,8 @@ ChannelFactory::ChannelFactory(const QDBusConnection &bus)
     setSubclassForOutgoingFileTransfers<OutgoingFileTransferChannel>();
     setSubclassForIncomingStreamTubes<IncomingStreamTubeChannel>();
     setSubclassForOutgoingStreamTubes<OutgoingStreamTubeChannel>();
+    setSubclassForIncomingRoomStreamTubes<IncomingStreamTubeChannel>();
+    setSubclassForOutgoingRoomStreamTubes<OutgoingStreamTubeChannel>();
     setSubclassForContactSearches<ContactSearchChannel>();
     setFallbackSubclass<Channel>();
 }
@@ -309,6 +311,52 @@ void ChannelFactory::setConstructorForIncomingStreamTubes(const ConstructorConst
     setConstructorFor(ChannelClassSpec::incomingStreamTube(QString(), additionalProps), ctor);
 }
 
+Features ChannelFactory::featuresForOutgoingRoomStreamTubes(const QVariantMap &additionalProps) const
+{
+    return featuresFor(ChannelClassSpec::outgoingRoomStreamTube(QString(), additionalProps));
+}
+
+void ChannelFactory::addFeaturesForOutgoingRoomStreamTubes(const Features &features,
+        const QVariantMap &additionalProps)
+{
+    addFeaturesFor(ChannelClassSpec::outgoingRoomStreamTube(QString(), additionalProps), features);
+}
+
+ChannelFactory::ConstructorConstPtr ChannelFactory::constructorForOutgoingRoomStreamTubes(
+        const QVariantMap &additionalProps) const
+{
+    return constructorFor(ChannelClassSpec::outgoingRoomStreamTube(QString(), additionalProps));
+}
+
+void ChannelFactory::setConstructorForOutgoingRoomStreamTubes(const ConstructorConstPtr &ctor,
+        const QVariantMap &additionalProps)
+{
+    setConstructorFor(ChannelClassSpec::outgoingRoomStreamTube(QString(), additionalProps), ctor);
+}
+
+Features ChannelFactory::featuresForIncomingRoomStreamTubes(const QVariantMap &additionalProps) const
+{
+    return featuresFor(ChannelClassSpec::incomingRoomStreamTube(QString(), additionalProps));
+}
+
+void ChannelFactory::addFeaturesForIncomingRoomStreamTubes(const Features &features,
+        const QVariantMap &additionalProps)
+{
+    addFeaturesFor(ChannelClassSpec::incomingRoomStreamTube(QString(), additionalProps), features);
+}
+
+ChannelFactory::ConstructorConstPtr ChannelFactory::constructorForIncomingRoomStreamTubes(
+        const QVariantMap &additionalProps) const
+{
+    return constructorFor(ChannelClassSpec::incomingRoomStreamTube(QString(), additionalProps));
+}
+
+void ChannelFactory::setConstructorForIncomingRoomStreamTubes(const ConstructorConstPtr &ctor,
+        const QVariantMap &additionalProps)
+{
+    setConstructorFor(ChannelClassSpec::incomingRoomStreamTube(QString(), additionalProps), ctor);
+}
+
 Features ChannelFactory::featuresForContactSearches(const QVariantMap &additionalProps) const
 {
     return featuresFor(ChannelClassSpec::contactSearch(additionalProps));
@@ -466,13 +514,13 @@ QString ChannelFactory::finalBusNameFrom(const QString &uniqueOrWellKnown) const
 }
 
 /**
- * Returns features as configured for the channel class given by the Channel::immutableProperties()
+ * Return features as configured for the channel class given by the Channel::immutableProperties()
  * of \a proxy.
  *
  * \todo Make the features configurable - currently an empty set is always returned.
  *
  * \param proxy The Channel proxy to determine the features for.
- * \return The channel class-specific features.
+ * \return A list of Feature objects.
  */
 Features ChannelFactory::featuresFor(const DBusProxyPtr &proxy) const
 {
