@@ -633,7 +633,7 @@ struct TELEPATHY_QT4_NO_EXPORT ClientRegistrar::Private
  *
  * \subsection cr_registering_sec Registering a client
  *
- * To register a client, just call registerClient with a given AbstractClientPtr
+ * To register a client, just call registerClient() with a given AbstractClientPtr
  * pointing to a valid AbstractClient instance.
  *
  * \code
@@ -652,6 +652,8 @@ struct TELEPATHY_QT4_NO_EXPORT ClientRegistrar::Private
  * \endcode
  *
  * \sa AbstractClientObserver, AbstractClientApprover, AbstractClientHandler
+ *
+ * See \ref async_model, \ref shared_ptr
  */
 
 /**
@@ -662,7 +664,7 @@ struct TELEPATHY_QT4_NO_EXPORT ClientRegistrar::Private
  * factory creating stock Telepathy-Qt4 channel subclasses, as appropriate, with no features ready.
  *
  * \param bus QDBusConnection to use.
- * \return A ClientRegistrarPtr object pointing to the ClientRegistrar.
+ * \return A ClientRegistrarPtr object pointing to the newly created ClientRegistrar object.
  */
 ClientRegistrarPtr ClientRegistrar::create(const QDBusConnection &bus)
 {
@@ -677,7 +679,7 @@ ClientRegistrarPtr ClientRegistrar::create(const QDBusConnection &bus)
  * \param connectionFactory The connection factory to use.
  * \param channelFactory The channel factory to use.
  * \param contactFactory The contact factory to use.
- * \return A ClientRegistrarPtr object pointing to the ClientRegistrar.
+ * \return A ClientRegistrarPtr object pointing to the newly created ClientRegistrar object.
  */
 ClientRegistrarPtr ClientRegistrar::create(
             const AccountFactoryConstPtr &accountFactory,
@@ -697,7 +699,7 @@ ClientRegistrarPtr ClientRegistrar::create(
  * \param connectionFactory The connection factory to use.
  * \param channelFactory The channel factory to use.
  * \param contactFactory The contact factory to use.
- * \return A ClientRegistrarPtr object pointing to the ClientRegistrar.
+ * \return A ClientRegistrarPtr object pointing to the newly created ClientRegistrar object.
  */
 ClientRegistrarPtr ClientRegistrar::create(const QDBusConnection &bus,
             const AccountFactoryConstPtr &accountFactory,
@@ -717,7 +719,7 @@ ClientRegistrarPtr ClientRegistrar::create(const QDBusConnection &bus,
  * AccountManager and AbstractClient implementations.
  *
  * \param manager The AccountManager the bus and factories of which should be used.
- * \return A ClientRegistrarPtr object pointing to the ClientRegistrar.
+ * \return A ClientRegistrarPtr object pointing to the newly ClientRegistrar object.
  */
 ClientRegistrarPtr ClientRegistrar::create(const AccountManagerPtr &manager)
 {
@@ -760,7 +762,7 @@ ClientRegistrar::~ClientRegistrar()
 /**
  * Return the D-Bus connection being used by this client registrar.
  *
- * \return QDBusConnection being used.
+ * \return A QDBusConnection object.
  */
 QDBusConnection ClientRegistrar::dbusConnection() const
 {
@@ -775,7 +777,7 @@ QDBusConnection ClientRegistrar::dbusConnection() const
  * situations where objects constructed at different times by the registrar would have unpredictably
  * different construction settings (eg. subclass).
  *
- * \return Read-only pointer to the factory.
+ * \return A read-only pointer to the AccountFactory object.
  */
 AccountFactoryConstPtr ClientRegistrar::accountFactory() const
 {
@@ -790,7 +792,7 @@ AccountFactoryConstPtr ClientRegistrar::accountFactory() const
  * situations where objects constructed at different times by the registrar would have unpredictably
  * different construction settings (eg. subclass).
  *
- * \return Read-only pointer to the factory.
+ * \return A read-only pointer to the ConnectionFactory object.
  */
 ConnectionFactoryConstPtr ClientRegistrar::connectionFactory() const
 {
@@ -805,7 +807,7 @@ ConnectionFactoryConstPtr ClientRegistrar::connectionFactory() const
  * situations where objects constructed at different times by the registrar would have unpredictably
  * different construction settings (eg. subclass).
  *
- * \return Read-only pointer to the factory.
+ * \return A read-only pointer to the ChannelFactory object.
  */
 ChannelFactoryConstPtr ClientRegistrar::channelFactory() const
 {
@@ -820,7 +822,7 @@ ChannelFactoryConstPtr ClientRegistrar::channelFactory() const
  * situations where objects constructed at different times by the registrar would have unpredictably
  * different construction settings (eg. subclass).
  *
- * \return Read-only pointer to the factory.
+ * \return A read-only pointer to the ContactFactory object.
  */
 ContactFactoryConstPtr ClientRegistrar::contactFactory() const
 {
@@ -828,11 +830,11 @@ ContactFactoryConstPtr ClientRegistrar::contactFactory() const
 }
 
 /**
- * Return a list of clients registered using registerClient() on this client
+ * Return the list of clients registered using registerClient() on this client
  * registrar.
  *
- * \return A list of registered clients.
- * \sa registerClient()
+ * \return A list of pointers to AbstractClient objects.
+ * \sa registerClient(), unregisterClient()
  */
 QList<AbstractClientPtr> ClientRegistrar::registeredClients() const
 {
@@ -861,7 +863,7 @@ QList<AbstractClientPtr> ClientRegistrar::registeredClients() const
  * \param clientName The client name used to register.
  * \param unique Whether each of a client instance is able to manipulate
  *               channels separately.
- * \return \c true if client was successfully registered, \c false otherwise.
+ * \return \c true if \a client was successfully registered, \c false otherwise.
  * \sa registeredClients(), unregisterClient()
  */
 bool ClientRegistrar::registerClient(const AbstractClientPtr &client,
@@ -955,6 +957,10 @@ bool ClientRegistrar::registerClient(const AbstractClientPtr &client,
         return false;
     }
 
+    if (handler) {
+        handler->setRegistered(true);
+    }
+
     debug() << "Client registered - busName:" << busName <<
         "objectPath:" << objectPath << "interfaces:" << interfaces;
 
@@ -972,7 +978,7 @@ bool ClientRegistrar::registerClient(const AbstractClientPtr &client,
  * If \a client was not registered previously, \c false will be returned.
  *
  * \param client The client to unregister.
- * \return \c true if client was successfully unregistered, \c false otherwise.
+ * \return \c true if \a client was successfully unregistered, \c false otherwise.
  * \sa registeredClients(), registerClient()
  */
 bool ClientRegistrar::unregisterClient(const AbstractClientPtr &client)
@@ -982,6 +988,12 @@ bool ClientRegistrar::unregisterClient(const AbstractClientPtr &client)
         return false;
     }
 
+    AbstractClientHandler *handler =
+        dynamic_cast<AbstractClientHandler*>(client.data());
+    if (handler) {
+        handler->setRegistered(false);
+    }
+
     QString objectPath = mPriv->clients.value(client);
     mPriv->bus.unregisterObject(objectPath);
     mPriv->clients.remove(client);
