@@ -92,21 +92,19 @@ PendingAccount::~PendingAccount()
 }
 
 /**
- * Return the AccountManagerPtr object through which the request was made.
+ * Return the account manager through which the request was made.
  *
- * \return An AccountManagerPtr object.
+ * \return A pointer to the AccountManager object.
  */
 AccountManagerPtr PendingAccount::manager() const
 {
-    return AccountManagerPtr(qobject_cast<AccountManager *>((AccountManager*) object().data()));
+    return AccountManagerPtr(qobject_cast<AccountManager *>((AccountManager*) _object().data()));
 }
 
 /**
- * Returns the newly created AccountPtr object.
+ * Return the newly created account.
  *
- * \return An AccountPtr object pointing to the newly create Account object, or
- *         a null AccountPtr if an error occurred.
- * \sa objectPath()
+ * \return A pointer to an Account object, or a null AccountPtr if an error occurred.
  */
 AccountPtr PendingAccount::account() const
 {
