@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2008 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2008 Nokia Corporation
+ * @copyright Copyright (C) 2008 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2008 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -52,15 +53,15 @@ public:
     ConnectionPtr connection() const;
 
 private Q_SLOTS:
-    void onCallFinished(QDBusPendingCallWatcher *watcher);
-    void onConnectionBuilt(Tp::PendingOperation *op);
+    TELEPATHY_QT4_NO_EXPORT void onCallFinished(QDBusPendingCallWatcher *watcher);
+    TELEPATHY_QT4_NO_EXPORT void onConnectionBuilt(Tp::PendingOperation *op);
 
 private:
     friend class ConnectionManagerLowlevel;
 
-    PendingConnection(const ConnectionManagerPtr &manager,
+    TELEPATHY_QT4_NO_EXPORT PendingConnection(const ConnectionManagerPtr &manager,
             const QString &protocol, const QVariantMap &parameters);
-    PendingConnection(const QString &error, const QString &errorMessage);
+    TELEPATHY_QT4_NO_EXPORT PendingConnection(const QString &error, const QString &errorMessage);
 
     struct Private;
     friend struct Private;
