@@ -82,19 +82,19 @@ PendingContactInfo::~PendingContactInfo()
 }
 
 /**
- * Return the ContactPtr object through which the request was made.
+ * Return the contact through which the request was made.
  *
- * \return A ContactPtr object.
+ * \return A pointer to the Contact object.
  */
 ContactPtr PendingContactInfo::contact() const
 {
-    return ContactPtr(qobject_cast<Contact*>((Contact*) object().data()));
+    return ContactPtr(qobject_cast<Contact*>((Contact*) _object().data()));
 }
 
 /**
- * Returns the information for contact().
+ * Return the information for contact().
  *
- * \return An object representing the contact information.
+ * \return The contact infor as a Contact::InfoFields object.
  */
 Contact::InfoFields PendingContactInfo::infoFields() const
 {
